*=======================================================================
* subroutine eliminate
*=======================================================================
*
* Identify snps in the same bin as the working_i snp
* Select the best snp from this bin
* Delete records from snpfile that are in the bin but not the best snp
*    in this bin.
* Delete records in correlate that refer to snps in this bin.
*
capture program drop eliminate
program define eliminate
*di "location x"
*codebook
*disp "------- beginning of eliminate --------------------------"
disp "working_i at beginning of eliminate = $working_i"

*qui {

preserve                  //

clear
qui set obs $correlate_N
qui getmata snp1 snp2 r r2 drop bin
qui replace drop=0
qui replace bin=0
qui replace bin=-1 if (snp1==$working_i | snp2==$working_i) & r2 >= $nextr2

sort bin snp1 snp2

local binlist ""
local newbinlist `binlist'
local identify=0
local j=0
*
* Initialize N to be the number of records in
* correlate.dta.  Don't want j to become
* bigger than N.
*
local N=_N
while `identify'==0 & `j'<`N'{
    local j=`j'+1
    if bin[`j']==-1 {
        *
        * add any new values of snp1 or snp2 to newbinlist
        *
        local s1 = string(snp1[`j'])
        local s2 = string(snp2[`j'])
        local newvalues=0
        if strpos("`newbinlist'","`s1'")==0 {
            local newvalues=1
            local newbinlist "`newbinlist' `s1'"
        }
        if strpos("`newbinlist'","`s2'")==0 {
            local newvalues=1
            local newbinlist "`newbinlist' `s2'"
        }
        *
        * If there are new values...
        *
        if `newvalues'==1 {
            *
            * For all records which have one of these new values
            * and for which r2>=$nextr2, set bin=-1.
            *
            qui replace bin=-1 if (snp1==`s1' | snp1==`s2') & r2>=$nextr2
            qui replace bin=-1 if (snp1==`s1' | snp2==`s2') & r2>=$nextr2
            local binlist `newbinlist'
            *di "newbinlist = `newbinlist'"
            sort bin snp1 snp2
        }
        else {
            *
            * Otherwise there are no new values so set
            * identify=1 and the while loop will terminate.
            *
            local identify=1
        }
    }
}
local j=1
local binlist ""
local snp_binlist ""

while bin[`j']==-1 {
    local s1 = string(snp1[`j'])
    local s2 = string(snp2[`j'])
    local binlist "`binlist' `s1' `s2'"
    local snp_binlist "`snp_binlist' snp_`s1' snp_`s2'"
    qui replace drop=1 if _n==`j'
    local j=`j'+1
}

qui drop if drop==1
*
* Remove duplicates from binlist.
*
local binlist : list uniq binlist
di "binlist = `binlist'"
local snp_binlist : list uniq snp_binlist
nois di "snp_binlist = `snp_binlist'"

global correlate_N = _N
*di "size of old correlate file = $correlate_N'"
qui putmata snp1 snp2 r r2 drop bin, replace

* nois disp "in eliminate, binlist = `binlist'"

if $nextr2==1 {
    clear
    qui set obs $snpfile_N
    qui getmata snps best
    qui gen drop=0
    *
    * Any snp will do as best in bin we will keep
    * the first snp in binlist
    *
    local i=0
    foreach s in `binlist'{
        local i=`i'+1
        replace drop=1 if `s'==snps & `i'>1
    }

    qui drop if drop==1
    qui drop drop
    global snpfile_N = _N
    qui putmata snps best, replace
*    qui save snpfile.dta, replace
}
else {
    *
    * Identify snp in binlist that best predicts cancer.
    * Drop the other snps in this list from snpfile.
    *
    clear
    qui set obs $case_control_N
    qui getmata case `snp_binlist'
    local bestvalue=0
    foreach snp of local binlist{
        local snpname "snp_`snp'"
        di "snpname = `snpname'"
        logit case `snpname', iter($nregiter)
        local x=max(0,abs(_b[`snpname'])-1.959964*_se[`snpname'])
        
* In order to choose the same snp accross platforms round to 6 significant figures
        
        local x= round(`x',10^( floor(log10(`x')) - 5))       
        if `x' >= `bestvalue' {
            local bestsnp `snp'
            local bestvalue=`x'
        }
    }
    di "bestvalue = `bestvalue' bestsnp = `bestsnp'"
    clear
    qui set obs $snpfile_N
    qui getmata snps best

    *
    * drop all snps in the curent bin that are not the
    * best unless the snp has already been selected with
    * a higher value of nextr2
    *
    * Remove bestsnp from binlist
    *
    local binlist=subinstr("`binlist'", "`bestsnp'", "", .)
    *
    * Remove from snpfile.dta the snps remaining in binlist.
    *
    di " binlist = `binlist'"

    foreach s of local binlist   {
        di " snps not bestvalue = `s' bestvalue = `bestvalue' bestsnp = `bestsnp'" 
        drop if snps==`s' & best ~= 1
    }

    global snpfile_N = _N
    qui putmata snps best, replace

*    qui save snpfile.dta, replace
    restore
}
*}  //quietly
* disp "------- end of eliminate --------------------------"
end // of routine eliminate
*=======================================================================
*
*=======================================================================
* subroutine bin
*=======================================================================
*
* This subroutine identifies bins of snps that are correlated with
* each other.  In each bin there must be one snp that has an
* r2 value >= nextr2 with every other snp in its bin.
*
capture program drop bin
program define bin
qui {		
disp "------- beginning of bin --------------------------"
di "location y"
*codebook
preserve
clear
di "snpfile_N = $snpfile_N"
qui set obs $snpfile_N
qui getmata snps best
*
* Set working_i to be the value of snps in the first record 
* of snpfile.dta.
*
global working_i=snps[1]
disp "working_i = $working_i"
*
* Loop while $working_i <= value of snps in the last record 
* of snpfile.dta.
*
while $working_i <= snps[_N] {
    eliminate
    clear
    qui set obs $snpfile_N
    qui getmata snps best
    *
    * set working_i to be smallest of snps that
    * is greater than the current working_i
    *
    qui gen working_i=$working_i
    qui gen greater=0
    qui replace greater=1 if snps>working_i
    qui egen tmpworking_i=min(snps) if greater ==1
    qui egen tmp2=max(tmpworking_i )
    global working_i=tmp2
    di "from bin next working i = $working_i"
    qui drop working_i greater tmpworking_i tmp2
}
*
* Initialise covariatelist.
*
local covariatelist ""
*
* Get the values of snps in a local macro "tmplist"
* For example if snps have the value of 2, 3, and 6
* then tmplist would be "2 3 6".
*
qui levelsof snps, local(tmplist)
*
* Make a covariatelist by prepending "snp_"
* to eace of the values in tmplist.  Given the 
* example in the comment above covariatelist 
* would be "snp_2 snp_3 snp_6".
*
foreach lname of local tmplist   {
    local x "snp_`lname'"
    local covariatelist `covariatelist' `x'
    }
*
* Add the values in $bestlist to covariatelist.
*
local covariatelist `covariatelist' $bestlist
*
* Make covariatelist contain unique values by
* removing all of one of duplicates.
*
local covariatelist : list uniq covariatelist
clear
qui set obs $case_control_N
qui getmata case `covariatelist'

disp "Regression at end of routine bin"

qui logit case `covariatelist', iter($nregiter)
if e(converged) ==  1 {
    *
    * Build bestlist global macro of SNPs for
    * the SNPs that survive the above regression.
    *
    global bestlist " "
    foreach snp of local covariatelist {
        local x=max(0,abs(_b[`snp'])-1.959964*_se[`snp'])
        local x= round(`x',10^( floor(log10(`x')) - 5))       
        if `x'>$bestbound & `x'~=. {
            global bestlist $bestlist `snp'
        }
    }
    di "bestlist = $bestlist
} 
else {
    *
    * If regression does not converge then
    * set bestlist to empty.
    *
    global bestlist " "
}
*
* At end of bin report the number of snps in
* covariatelist and $bestlist
*
} 
if $n_boot == 0 {
    disp "Local covariatelist contains " wordcount("`covariatelist'") " snps."
    disp "Global bestlist contains " wordcount("$bestlist") " snps."
    disp "bestlist snps = $bestlist"
}
*
* Set best=1 for each snp in bestlist.
*
clear
qui set obs $snpfile_N
qui getmata snps best
qui replace best=0
foreach snp of global bestlist {
    local snpnum=real(subinstr("`snp'", "snp", "", 1))
    qui replace best=1 if snps==`snpnum'
} 
global snpfile_N = _N
qui putmata snps best, replace
*qui save snpfile.dta, replace
restore

*disp "------- end of bin --------------------------"
end //of routine bin
*=======================================================================
* subroutine sentinel
*=======================================================================
capture program drop sentinel
program define sentinel
*
* Inputs
*    $working_cancerfile - cancer data set.
*    delta - a stepsize used to decrement the value of r2
*        that defines bin sizes
*    bestbound - the smallest lower 95% CI boud for the log
*        odds ratio of risky snps that are assigned best=1.
*        - bestbound = the largest upper CI bound for protective snps
*        that are assigned best=1.  snps that are not assigned
*        best=1 have 95% CIs that extend into the interval
*        +/- bestbound.
*
*
* Output
*    bestlist - a list of snps that are significant in a
*        multivariate model and which include important
*        snps that might have otherwise have been deleted
*        due to high linkage disequilibrium.
*
* nextr2 - the r2 value used to identify snps in the same bin.
*
* =============================================================
* make correlate.dta
*disp "Building correlate.dta"
* =============================================================
*
* Make a file "correlate" that contains 3 variables:
*    snp1 - integer snp name and takes values 1,2,...n
*    snp2 - integer snp name and takes values 2,3,...n
*    r2 - squared correlation coefficient between snp1 & snp2
*    drop - indicates if record should be dropped.  
*          Set in function eliminate.
* This file contains (n*(n-1))/2 records and gives the pearson correlation
* coefficient for each possible pair of snps.
*
* Read in the main data set again.
*
* Local macro n contains count of SNP variables.
*
*codebook
quietly ds snp_*
local n=wordcount(r(varlist))
*
* snpvars is a list of the SNP variables names.
*
unab snpvars: snp_*
*
* Create and open correlate.csv.
*
preserve

file open myfile using "correlate.csv", write replace
file write myfile  "snp1, snp2, r, r2, drop" _n
*
* Read in the main data set again.
*
restore
*
* Local macro n contains count of SNP variables.
*
quietly ds snp_*
local n=wordcount(r(varlist))

*
* Loop over each unique pair of snps and calculate correlation.
*
disp "Building correlate.dta"
disp "...processing `n' SNPs"
*
forvalues i=1/`n' {
    local xx=`i'+1
    forvalues j=`xx'/`n' {
        qui pwcorr snp_`i' snp_`j'
        local r=r(rho)
            local r2=r(rho)*r(rho)
            file write myfile "`i' , `j', `r', `r2', 0" _n
    }
}
file close myfile
*
* Convert correlate.csv into correlate.dta.
* Write out a data set of SNP pairs and R**2 value.
* correlate is initially sorted by snp1 & snp2
*
preserve
 qui import delimited correlate.csv, clear 
gen bin=0
sort snp1 snp2

* Remember the size of this file and save the variables as vectors

global correlate_N = _N
qui putmata snp1 snp2 r r2 drop bin, replace

*==========================================================================
* Create snpfile.dta, which is an initial list fot the numerical snp names.
*===========================================================================
*disp "Building snpfile.dta"
*
* Read in the main data set.
*
restore
*
* Local macro n contains count of SNP variables.
*
quietly ds snp*
local n=wordcount(r(varlist))
disp "The dataset contains `n' SNPs"
*
* snpvars is a list of the SNP variables names.
*
unab snpvars: snp*
*
* Make a file snpfile that contains two variables: snps, best.
*
preserve
clear
quietly set obs `n'
gen snps = _n
gen best = 0
global snpfile_N = _N
qui putmata snps best, replace

*qui save "snpfile.dta", replace

restore
quietly ds snp*
local inputn=wordcount(r(varlist))
global nextr2=1
*
* bestlist - this will be a list of snps that predict cancer well.
*        Initially null.
*
global bestlist " "

*disp "delta = $delta"
*disp "bestbound = $bestbound"

forvalues k=1/$iter_end {
    if $n_boot == 0 {
        di "k = `k' delta = $delta iter_end = $iter_end"
    }
    bin
*    di "bin return"
    global nextr2=$nextr2-$delta
}
*
* Make a local macro by copying the global
* $bestlist.
*
local covariatelist $bestlist
*
* Run a regression with the selected SNPs as the
* covariates.
*
qui logit case `covariatelist', iter($nregiter)
*
* Remove the SNPs that were dropped by the above
* regression.  Make a list of the remaining SNPs.
*
global bestlist ""
foreach snp of local covariatelist {
    local x=max(0,abs(_b[`snp'])-1.959964*_se[`snp'])
    local x= round(`x',10^( floor(log10(`x')) - 5))       
    if `x'>$bestbound {
        global bestlist $bestlist `snp'
    }
}

local tempn=wordcount("`covariatelist'")
local bestn=wordcount("$bestlist")

*
* Run a final regression with the final snp list
* in the model.
*
*logit case $bestlist, iter($nregiter)
*
* Run a pwcorr on SNPs in bestlist.
*
*if `bestn' > 1  pwcorr $bestlist
*disp "------- end of sentinel --------------------------"
end // of routine sentinel

*************************************************************************
