
local stubname ds_20180119
local prefix sentinelSNPs.matrix.new.public.JRSboot.delta.025.
log using `prefix'`stubname'.log, replace
display "`prefix'`stubname'.log"

* Bootstrap the sentinel program on `stubname'.dta . This program looks
* for `stubname'.dta in the parent folder.

* This program is designed to run a bootstrap on the public data set using a bootstrapped
* sample equal to that of the JRS data set.

global nregiter=15  // iteration limit for regressions
*
* Set n_boot = number of bootstrap interations. Set n_boot=0 if no bootstrapping
* is wanted.
*
global n_boot=1

local localseed=`1'
disp "localseed=`localseed'"
set seed `localseed'
set matsize 1000

global delta=0.025 // a stepsize used to decrement the value of R^2 used in 
                 // the sentinel program                 
global iter_end=1/$delta // = the number of different values of $nextr2 
                         //   evaluated by the bin program
                 
* bestbound = smallest lower bound of the 95% CI for the adjusted logodds
*             of risky snps that are considered for inclusion in the final model. 
*             For protective snps, - bestbound is the largest upper bound for
*             the 95% CI for the adjusted logodds that are considered for inclusion.
global bestbound=log(1.1)  


* The program reduces I/O to temporary datasets

* N.B. The original version of this program wrote temporary data to 
* correlate.dta and snpfile.data.  These files have been replaced by
* mata vectors to reduce I/O. The comments below, however, still refer
* to these temporary files.

include include_sentinel.new.do

* SSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSS

* Start of program

* SSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSS


*pause on
set more off

*
* This program expects its input file to contain variables
* case, snp1, snp2, ..., snpn
*
* If the input cancer file to be processed is not in this
* configuration it will have to be converted.  The first code
* below does that conversion for a test data set called
* "test_data-with-realistic-names.dta".  This file is converted
* into a file called "test_data-mappednames.dta". The genetic
* variants in "test_data-with-realistic-names.dta" must start with
* the letters "snp".
*
* "test_data-mappednames.dta" is the data set that is actually
* processed.
*
* The conversion process also produces a file "name_map.dta"
* that allows us to map from the snp1, snp2, ..., snpn names
* back to the original names.
*
* ################################################
* Define the cancer file.  Set a global macro
* that contains the name of the cancer file.
*
*==== v SET INPUT FILE NAME HERE v =========================
global input_data "../../../`stubname'.dta"
*==== ^ SET INPUT FILE NAME HERE ^ =========================
*
*==== v DO NOT CHANGE THESE NAMES v ========================
global processed_input_data "processed_input_data.dta"
global cancerfile_mapped "test_data-mappednames.dta"
*==== ^ DO NOT CHANGE THESE NAMES ^ ========================
*
* One can do some minor manipulation of the input
* file if desired.
*
* At a minimum, in order to get the data into correctly named file,
* execute: 
* "use $input_data" & "save $processed_input_data, replace"
*
use "$input_data"
keep if study==1 // public patients only
gen case=case_caco_hpc
include "622niceSNPs.txt"
keep case `legit_snps'
drop if case == .
logit case snp*

qui ds snp*
local snplist `r(varlist)'

*    preserve
*        gen patients=1
*        collapse (count) patients, by(case `snplist')
*        list
*    restore

save $processed_input_data, replace
*
* ################################################
*
* ################################################
* Make a data set where the SNP names are are renamed
* to be snp1, snp2, etc.
*
file open myfile using "name_map.csv", write replace
file write myfile  "n, real_snpname, abbre_snpname" _n
disp "Building name_map.dta"
use $processed_input_data, clear 
qui ds snp*
local spncount=wordcount(r(varlist))
local snplist `r(varlist)'

* Analyze the complete data

qui logit case `snplist'
predict logodds, xb
roctab case logodds
*
* Rename the case variable to "case".
*
* rename realcasename case
*
* Rename the SNP variable names to snp_1, snp_2, ...
* Create a local macro with these names.
*
local i=0
local complete_snp_list = " "
foreach snpname of local snplist {
    local i=`i'+1
    rename `snpname' snp_`i'
    local complete_snp_list = "`complete_snp_list' snp_`i'"
           file write myfile "`i', `snpname' , snp_`i'" _n
}
file close myfile
dis "Complete snp list = `complete_snp_list'"

*
* Save the analysis data set with the new names.
*
save $cancerfile_mapped, replace
save working_cancerfile, replace
save real_cancerfile_with_simple_snp_names, replace
global case_control_N = _N
putmata *
preserve
import delimited name_map.csv, varnames(1) clear
save name_map.dta, replace
restore

sentinel

*  output time here.

display "$S_TIME  $S_DATE"

*
* Translate back to real snp names.
*
use "name_map.dta" 
gen use=0
foreach snp of global bestlist {
    qui replace use=1 if trim(abbre_snpname) == trim("`snp'")
}
qui keep if use==1
keep real_snpname

local bestrealnames " "
local N=_N
forvalues i = 1/`N' {
    local snp=real_snpname[`i']
    local bestrealnames "`bestrealnames' `snp'"
}
local bestrealnames=trim("`bestrealnames'")

local bestn=wordcount("`bestrealnames'")

disp "sentinel has selected `bestn' SNPs. They are: $bestlist" //edit

disp "delta = $delta"
disp "bestbound = $bestbound"
*
use $processed_input_data, clear
logistic case `bestrealnames', iter($nregiter)
if `bestn' > 1 pwcorr `bestrealnames'
predict logodds2, xb
roctab case logodds2

*BBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBB
*
* Now do the bootstrap analyses  //not edited
*
* BBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBB

if $n_boot > 0 {

* Make an empty p_values.`prefix'`stubname'.dta data set.
* Define a p-value variable for each snp. 
*
    use real_cancerfile_with_simple_snp_names, clear
    ds snp_*
    local all_snps `r(varlist)'
*di "`all_snps'"
    clear
    set obs 0
    local snps=subinstr("`all_snps'","n_","",.)
    foreach snp in `snps' {
        quietly gen p_`snp' = .
    }
    foreach snp in `snps' {
        quietly gen coef_`snp' = .
    }
    quietly gen AUCbootData = .
    quietly save p_values.`prefix'`stubname'.dta, replace 
    ds
* ===============================================================

global bootstrapped_data "bootstrapped_data.dta"

*
* Run the procedure on bootstrapped data.
*
global case_control_N = 1154 // = JRS sample size

    display "Number of simulations = $n_boot"
    nois _dots 0, title(Loop running) reps($n_boot)
    forvalues i = 1/$n_boot {
        nois _dots `i' 0
        use real_cancerfile_with_simple_snp_names, clear
*
* Create the bootstrap sample.
*
        bsample ,strata(case)
        *
        * Reduce the data set to Smith size with appropriate
        * numbers of cases and controls.
        *
	count
	gen rand = runiform()
	sort case rand
	by case:gen keep = _n
	drop if case == 0 & keep > 823
	drop if case == 1 & keep > 331
	drop rand keep
	count
	di "bootstrapped sample size = " _N

        quietly save bootstrapped_sample, replace
        global working_cancerfile "bootstrapped_sample"
        
        quietly putmata *, replace
        quietly sentinel
*    disp "bestlist (returned by sentinel): $bestlist"
*    
*    For each snp that is included in the resultant model as determined by
*    the sentinel program, save the p-value in a scalar variable.
*    If the snp does not end up in the model then save a 1.
*    Also save each snp coefficient. Set equal to zero for snps not in model.
*    Finally, calculate a classifier variable equal to the estimated logodds
*    minus the constant term from the logistic regression based on the snps
*    selected by sentinel.
*
        qui gen classifier = 0
        foreach snp of varlist `all_snps' {
            novarabbrev capture :  di _b[`snp']
            if _rc == 0 {
                 scalar _Sp_`snp' = 2*normal(-abs(_b[`snp']/_se[`snp']))
                 scalar _Scoef_`snp' = _b[`snp']
            }
            else {
                 scalar _Sp_`snp' = 1 
                 scalar _Scoef_`snp' = 0
            }
            qui replace classifier = classifier + _Scoef_`snp'*`snp'
            *disp "scalar _Sp_`snp' = " _Sp_`snp'
        }
        qui roctab case classifier
        local AUC = r(area)
*        di "`AUC'"
*    
*     Save the p-values from the regression in the i-th row of 
*     p_values.`stubname'.dta (created above)   
*
        use p_values.`prefix'`stubname'.dta, clear
        quietly set obs `i'
*       di " AUC = `AUC'"
        qui replace AUCbootData = `AUC' if _n == `i'
        foreach snp of varlist p_snp* {
            local tmp=subinstr("`snp'","p_","",1)
            novarabbrev capture :  di _Sp_`tmp'
            if _rc == 0 {
                qui replace `snp' = _Sp_`tmp' if _n == `i'
            }
        }
        foreach snp of varlist coef_snp* {
            local tmp=subinstr("`snp'","coef_","",1)
            novarabbrev capture :  di _Scoef_`tmp'
            if _rc == 0 {
                qui replace `snp' = _Scoef_`tmp' if _n == `i'
            }
        }
        quietly save, replace

*    disp " End of simulation `i'"
*    disp " ======================================================="
        }

* Restore the original SNP names in p_values.`prefix'`stubname'.dta

    use p_values.`prefix'`stubname'.dta, clear
*list

    local i = 0
    foreach snpname of local snplist {
        local i = `i' + 1
        qui rename p_snp_`i' p_`snpname' 
        }
    save ,replace
*    list p_*
*    list coef* AUCbootData
    
*List the proportion of bootstrapped samples with p < 0.05 for each snp
    di "`snplist'"
    foreach snpname of local snplist {
*        di "`snpname'"
        qui gen `snpname'_in = 0
        qui replace `snpname'_in = 1 if p_`snpname' < 0.05
        }
    sum *_in
}
log close
